/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.field;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.FieldManager;
import nl.cloudfarming.client.field.model.Shape;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ui.support.NodeFactory;
import org.netbeans.spi.project.ui.support.NodeFactorySupport;
import org.netbeans.spi.project.ui.support.NodeList;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.nodes.NodeTransfer;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author Timon Veenstra
 */
@NodeFactory.Registration(projectType = "nl-cloudfarming-client-field")
@NbBundle.Messages({"FieldsNode.name=Fields", "ImportNode.name=Import", "HelpLinesNode.name=Help Lines"})
public class FieldProjectNodeFactory implements NodeFactory {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field");

    @Override
    public NodeList<?> createNodes(Project p) {
        List<Node> nodes = new ArrayList<>();
        // add fields node to project
        FileObject fieldsFolder = ((FieldProject) p).getFieldsFolder(true);
        DataFolder fieldsDataObject = DataFolder.findFolder(fieldsFolder);
        nodes.add(new FieldsNode(fieldsDataObject.getNodeDelegate()));
        // add import node to project
        FileObject importFolder = ((FieldProject) p).getImportFolder(true);
        DataFolder importDataObject = DataFolder.findFolder(importFolder);
        nodes.add(new ImportNode(importDataObject.getNodeDelegate()));
        // add help lines node to project
        FileObject helpLinesFolder = ((FieldProject) p).getHelpLinesFolder(true);
        DataFolder helpLinesDataObject = DataFolder.findFolder(helpLinesFolder);
        nodes.add(new HelpLinesNode(helpLinesDataObject.getNodeDelegate()));

        return NodeFactorySupport.fixedNodeList(nodes.toArray(new Node[nodes.size()]));
    }

    private static final class FieldsNode extends FilterNode {

        public FieldsNode(Node original) {
            super(original, new FilterNode.Children(original), new ProxyLookup(original.getLookup(), Lookups.fixed(new SimpleLayer(Bundle.FieldsNode_name()))));
            disableDelegation(DELEGATE_SET_NAME);
            setName(Bundle.FieldsNode_name());
        }

        @Override
        public boolean canCopy() {
            return false;
        }

        @Override
        public boolean canRename() {
            return false;
        }

        @Override
        public boolean canCut() {
            return false;
        }

        @Override
        public PasteType getDropType(final Transferable t, int action, int index) {
            if (t.isDataFlavorSupported(Shape.DATA_FLAVOR)) {
                return new PasteType() {

                    @Override
                    public Transferable paste() throws IOException {
                        try {
                            Shape shape = (Shape)t.getTransferData(Shape.DATA_FLAVOR);
                            FileObject fieldsFolder = getLookup().lookup(FileObject.class);
                            String name = FileUtil.findFreeFileName(fieldsFolder, shape.getName(),Field.EXT);
                            LOGGER.log(Level.FINEST, "found free filename: {0}", name);
                            FileObject fieldFile = fieldsFolder.createData(name,Field.EXT);
                            
                            FieldManager fieldManager = Lookup.getDefault().lookup(FieldManager.class);
                            fieldManager.saveAsField(shape, fieldFile);
                        } catch (UnsupportedFlavorException ex) {
                            Exceptions.printStackTrace(ex);
                        }
                        return null;
                    }
                };
            }
            return null;
        }
        
        @Override
        public Transferable drag() throws IOException {
            return clipboardCopy();
        }

        @Override
        public Transferable clipboardCopy() throws IOException {
            return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
        }        
    }

    private static final class HelpLinesNode extends FilterNode {

        public HelpLinesNode(Node original) {
            super(original, new FilterNode.Children(original), new ProxyLookup(original.getLookup(), Lookups.singleton(new SimpleLayer(Bundle.HelpLinesNode_name()))));
            disableDelegation(DELEGATE_SET_NAME);
            setName(Bundle.HelpLinesNode_name());
            disableDelegation(DELEGATE_GET_ACTIONS);
        }

        @Override
        public boolean canCopy() {
            return false;
        }

        @Override
        public boolean canRename() {
            return false;
        }

        @Override
        public boolean canCut() {
            return false;
        }
    }

    private static final class ImportNode extends FilterNode {

        public ImportNode(Node original) {
            super(original, new ImportNodeChildren(original));
            disableDelegation(DELEGATE_SET_NAME);
            setName(Bundle.ImportNode_name());
            disableDelegation(DELEGATE_GET_ACTIONS);
        }

        @Override
        public boolean canCopy() {
            return false;
        }

        @Override
        public boolean canRename() {
            return false;
        }

        @Override
        public boolean canCut() {
            return false;
        }
    }

    public static class ImportNodeChildren extends FilterNode.Children {

        public ImportNodeChildren(Node node) {
            super(node);
        }

        @Override
        protected Node[] createNodes(Node object) {
            List<Node> result = new ArrayList<>();

            for (Node node : super.createNodes(object)) {
                if (accept(node)) {
                    result.add(node);
                }
            }

            return result.toArray(new Node[0]);
        }

        private boolean accept(Node node) {
            //TODO figure out how to filter out secondary files
            return true;
        }
    }
}
